/*
 * Copyright 2009-2015 Dovetailed Technologies, LLC.
 *
 * Licensed under the accompanying Co:Z Target System Toolkit Source License Agreement (the "License");
 * you may not use this file except in compliance with the License.
 *
 */

#ifndef _COZ_UTIL_H
#define _COZ_UTIL_H

class CoZUtil {
	
	public:

 	 	/* Parse command into individual arguments, tokenized by spaces or
 	 	 * single quotes.
	 	 * On entry to this function, args is assumed to be allocated and argc set
	 	 * to the size of the allocated argument array.  
	 	 * On exit, args will contain the tokenized arguments and argc will be
	 	 * set to the number of arguments parsed.  In addition, args[argc] will be
	 	 * set to NULL.  The command argument WILL be modified.
	 	 */
		static void parseCommand(char* command, char** args, int& argc) {
			TRACE_CTX("parseCommand");

			int max_args = argc;
			argc = 0;
			char* p = command; // point to first char of command
			for (;;) {
				char* q;
				while (*p && isblank(*p)) p++;
				if (!*p)
					break;
				if (argc >= max_args) {
					LOG(LOG_ERR, "Error: too many command tokens");
					throw(RC_INPUT_ERROR);
				}
				if (*p == '\'') {  // quoted token
					for (q = ++p; *q && *q != '\''; q++);
				} else { // space terminated token
					for (q = p; *q && !isblank(*q); q++);
				}
				args[argc] = p;
				if (*q) *q++ = '\0';
				LOG(LOG_DEBUG, "arg[%d] = \'%s\'", argc, p);
				argc++;
				p = q;
			}	
			args[argc] = NULL;  // null terminate args list
			TRACE_CTX_RETVAL("nargs=%d", argc);
		}
	
		static char* newStringCopy(const char* s) {
			if (s == NULL) return NULL;
			char* answer = new char[strlen(s)+1];
			strcpy(answer, s);
			return answer;
		}

		static void assignString(char** target, const char* source) {
			if (*target) delete[] *target;
			*target = newStringCopy(source);
		}
		
		static char* trim(char* string) {
			if (!string) return NULL;
			char* p = string;
			while (*p && isspace(*p++))
				;
			p--;
			char* q = p + strlen(p) - 1;
			while (q>=p && isspace(*q)) 
				*(q--) = '\0';
			return p; 
		}
				
};

#endif  /* _COZ_UTIL_H */
