/*
 * Copyright 2009-2015 Dovetailed Technologies, LLC.
 *
 * Licensed under the accompanying Co:Z Target System Toolkit Source License Agreement (the "License");
 * you may not use this file except in compliance with the License.
 *
 */

#ifndef ElapsedTime_H_
#define ElapsedTime_H_


#include <sys/time.h>
#include <time.h>

class ElapsedTime {
private:
	timeval start_time;
    clock_t start_clock;
    
public: 
    timeval cumm_time;
    timeval cumm_cpu;
    
    timeval lap_time;
    timeval lap_cpu;
    
    unsigned long lapCount;
    
	ElapsedTime() : lapCount(0) {
		
		reset();
	}
	
	void reset() { // resets cumm times and start time
		cumm_time.tv_sec = 0;
		cumm_time.tv_usec = 0;
		cumm_cpu.tv_sec = 0;
		cumm_cpu.tv_usec = 0;
		lapCount = 0;
		start();
	}
		
	void start() {
		gettimeofday(&start_time, NULL);
		start_clock = clock();			
		lapCount++;
	}

	// update lap_time and lap_cpu, but not lapCount or cumm_*
	void update() {
		gettimeofday(&lap_time, NULL);
		clock_t now = clock();

		subtract(lap_time, start_time);

		now -= start_clock;
		lap_cpu.tv_sec = now / CLOCKS_PER_SEC;
		lap_cpu.tv_usec = now % CLOCKS_PER_SEC;
	}
	
	void stop() {
		gettimeofday(&lap_time, NULL);
		clock_t now = clock();

		subtract(lap_time, start_time);
		add(cumm_time, lap_time);

		now -= start_clock;
		lap_cpu.tv_sec = now / CLOCKS_PER_SEC;
		lap_cpu.tv_usec = now % CLOCKS_PER_SEC;
		add(cumm_cpu, lap_cpu);
	}
	
	unsigned long getLapMillis() {
		return lap_time.tv_sec*1000 + lap_time.tv_usec/1000;
	}

	unsigned long getLapCpuMillis() {
		return lap_cpu.tv_sec*1000 + lap_cpu.tv_usec/1000;
	}

	unsigned long getCummMillis() {
		return cumm_time.tv_sec*1000 + cumm_time.tv_usec/1000;
	}
	
private:
	
	void subtract(timeval& t1, timeval& t2) {
		t1.tv_sec -= t2.tv_sec;
		t1.tv_usec -= t2.tv_usec;
		if (t1.tv_usec < 0) {
			t1.tv_usec += 1000000;
			t1.tv_sec--;
		}
	}
	
	void add(timeval& t1, timeval& t2) {
		t1.tv_sec += t2.tv_sec;
		t1.tv_usec += t2.tv_usec;
		if (t1.tv_usec >= 1000000) {
			t1.tv_usec -= 1000000;
			t1.tv_sec++;
		}
	}
		
				
};

#endif /* ElapsedTime_H_ */
